-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 1999,2005 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive/db/sql/procs/al_inserteventteclongattr.sql, tbsm.sql, tbsm_base31e
-- SID: 1.2
-- Date Modified: 07/01/09 10:31:48
--
-- Module Description
--   Defines the al_insertEventTecLongAttr stored procedure.

USE Object
GO

--------------------------------------------------------------------------------
-- asisp_getEventTecLongAttr
PRINT 'Creating procedure al_insertEventTecLongAttr'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'al_insertEventTecLongAttr' AND type = 'P')
  DROP PROCEDURE al_insertEventTecLongAttr
GO

-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 1999,2005 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive/db/sql/procs/al_inserteventteclongattr.sql, tbsm.sql, tbsm_base31e
-- SID: 1.2
-- Date Modified: 07/01/09 10:31:48

--------------------------------------------------------------------------------
-- This procedure inserts a row in to the ExtendedEventTecLongAttr table.  It
-- assumes that the caller has broken down long attributes into sequenced
-- substrings, based on the length of the ExtendedEventTecLongAttr.value column.
--
-- Callers:  AlJdbcBatchSet.java
--------------------------------------------------------------------------------
CREATE PROCEDURE al_insertEventTecLongAttr
  @event_handle    NVARCHAR(16),
  @server_handle   NVARCHAR(16),
  @date_reception  NVARCHAR(16),
  @event_class     NVARCHAR(255),
  @seq             INT,
  @keyword         NVARCHAR(64),
  @value           NVARCHAR(3600),
  @trId            INT = 0
AS
  -- Begin transaction
  DECLARE @tcount INT
  SET @tcount = @@TRANCOUNT
  IF @tcount = 0
    BEGIN TRANSACTION al_insertEventTecLongAttr
  ELSE
    SAVE TRANSACTION al_insertEventTecLongAttr

  IF EXISTS (SELECT * FROM ExtendedEventTecLongAttr WITH (NOLOCK)
               WHERE event_handle = @event_handle
                 AND server_handle = @server_handle
                 AND date_reception = @date_reception
                 AND event_class = @event_class
                 AND seq = @seq
                 AND keyword = @keyword)
  BEGIN
    -- A duplicate event (defined as an event with the same event_handle,server_handle,date_reception, and event_class)
    -- can be received from TEC, particularly in the case of an update to a previous event.  
      DELETE FROM ExtendedEventTecLongAttr
        WHERE event_handle = @event_handle
          AND server_handle = @server_handle
          AND date_reception = @date_reception
          AND event_class = @event_class
          AND keyword = @keyword

  END

  BEGIN
    -- Insert new event and associated long attribute
    INSERT INTO ExtendedEventTecLongAttr (event_handle, server_handle, date_reception, event_class, seq, keyword, value)
      VALUES(@event_handle, @server_handle, @date_reception, @event_class, @seq, @keyword, @value)
  END

  -- Commit transaction
  IF @tcount = 0
    COMMIT TRAN cl_preprocessTECResources

GO
