-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 1999,2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive/db/sql/procs/hist_classevents.sql, tbsm_base, tbsm_base31e
-- SID: 1.6.4.7
-- Date Modified: 07/02/21 14:48:01
--
-- Module Description
--   This procedure is used to produce the Class Events report for TBSM.
--

USE Object
GO

----------------------------------------------------------------------------------------------------
-- hist_ClassEvents
PRINT 'Creating procedure hist_ClassEvents'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'hist_ClassEvents' AND type = 'P')
  DROP PROCEDURE hist_ClassEvents
GO

-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 1999,2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive/db/sql/procs/hist_classevents.sql, tbsm_base, tbsm_base31e
-- SID: 1.6.4.7
-- Date Modified: 07/02/21 14:48:01
--
-- Module Description
--   This procedure is used to produce the Class Events report for TBSM.
--

CREATE PROCEDURE hist_ClassEvents
  @obj_cid    ClassID,
  @ecid       NVARCHAR(255),
  @alertstate NVARCHAR(3),
  @priority   NVARCHAR(3),
  @begtime    VARCHAR(30),
  @endtime    VARCHAR(30),
  @detail     VARCHAR(255),
  @extended_length_attr_filter NVARCHAR(255) -- DCR117
AS

  -- dcr117 changed @whereClause and @whereClause2 from 255 to 500 in case detail search or extended attribute has 255 characters
  DECLARE @whereClause VARCHAR(500) 
  DECLARE @whereClause2 VARCHAR(500)
  DECLARE @tableClause VARCHAR(255)
  DECLARE @tableName ObjName

  SELECT @tableName = cname+"_V" FROM obj_class WHERE cid=@obj_cid

  SELECT @whereClause = "e.obj_cid = '" + RTrim(@obj_cid) + "'"

  IF @ecid <> "" 
    SELECT @whereClause = @whereClause + " AND e.cid IN(" + @ecid + ")"

  IF @alertstate <> ''
    SET @whereClause = @whereClause + ' AND e.alertstate=' + @alertstate

  IF @priority <> ''
    SET @whereClause = @whereClause + ' AND e.priority=' + @priority

   SELECT @whereClause = @whereClause + " AND e.ctime >= CONVERT(DATETIME,'" + @begtime + "',101)" 
  SELECT @whereClause = @whereClause + " AND e.ctime < CONVERT(DATETIME,'" + @endtime + "',101) " 
  IF @detail <> "" 
    SELECT @whereClause2 = " AND e.detail LIKE '" + @detail + "' "
    
  -- DCR117, the event search string should be converted to sql form before passing in
  -- For example, if a user enters *tec event * into the search input in report,
  -- the search string must be converted to %tec event% and pass this converted string into this procedure
  IF @extended_length_attr_filter <> ""
     SELECT @whereClause = @whereClause + " AND e.extended_attr LIKE '" + @extended_length_attr_filter + "' "

  SELECT @tableClause = " and c.id=e.obj_id "

  -- OA19932: since event table on primary db server does not have the extended_attr column (DCR 117), 
  -- need to send in "" for it for customers running reporting system solely off primary db server
  DECLARE @tableid INT
  SELECT @tableid = id FROM sysobjects WHERE name = 'event'
  IF EXISTS (SELECT * FROM syscolumns WHERE id = @tableid AND name = 'extended_attr')
    EXEC('SELECT 
        e.obj_cid objcid,
        c.name objname,
        ast._Desc alertstate,
        op._Priority priority,
        e.name msgname,
        e.detail msgtext,
        e.ctime datetime,
        e.extended_attr
      FROM
        event e, ' +
        @tableName + ' c,
        SixLevelPri_A op,
        AlertState_A ast
      WHERE ' + @whereClause + @tableClause + @whereClause2 + ' AND e.priority = op.id AND e.alertstate = ast.id ORDER BY e.ctime')
  ELSE
    EXEC('SELECT 
        e.obj_cid objcid,
        c.name objname,
        ast._Desc alertstate,
        op._Priority priority,
        e.name msgname,
        e.detail msgtext,
        e.ctime datetime,
        ""
      FROM
        event e, ' +
        @tableName + ' c,
        SixLevelPri_A op,
        AlertState_A ast
      WHERE ' + @whereClause + @tableClause + @whereClause2 + ' AND e.priority = op.id AND e.alertstate = ast.id ORDER BY e.ctime')
