-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                     
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 1999,2003 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive/db/sql/procs/hist_lobevents.sql, tbsm_base, tbsm_base31e
-- SID: 1.10.6.15
-- Date Modified: 07/02/21 14:48:02
--	
-- Module Description:
--   Defines the hist_LobEvents stored proc which is used by the Reporting System.
-- History of Changes:
--   OW57191 - LOB REPORTS TIMEOUT FROM REPORTING SYSTEM
--   117022  - Report "hangs" when mach/sys column is null.
--   OA09872 - Corrections to properly report on deleted objects.
--             Support includeDeleted option passed from report.
--   OA11656 - When calling report_log_objects, the temp table #obj_temp now icluded a integer column
--             named "deleted'. This column will now contain a value of 1 if the resource is deleted.
--             The column value will be used to prefix resource names with a "*" for the report this
--             procedure supports. This was done to eliminate a Join to the link table which was returning
--             duplicated resources when a resource had both a deleted and undeleted link row.
--   PMR25706 - The mach and name column in #objtemp was increased in size from ObjName - NVARCHAR(54) to NVARCHAR(255)
--              This was required for deleted resources with names that were already at the max length of 54.
--              Attempting to pre-pend the '*' character onto deleted resource name would generate a "String
--              or bianry data would be truncated" error in the Reporting System WEB application.
--   DCR117   - query extended length attribute and do event search filter if event search text is not empty
--   OA18672  - changed the whereclause for detail & msgname to what it was before DCR 117 and
--              added a whereclause for extended_length_attr_filter similar to detail
USE Object
GO

-------------------------------------------------------------------------------------------
PRINT 'Creating procedure hist_LobEvents.'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'hist_LobEvents' AND type = 'P')
  DROP PROCEDURE hist_LobEvents
GO

-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                     
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 1999,2003 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive/db/sql/procs/hist_lobevents.sql, tbsm_base, tbsm_base31e
-- SID: 1.10.6.15
-- Date Modified: 07/02/21 14:48:02
--
-- Module Description:
--   Defines the hist_LobEvents stored proc which is used by the Reporting System.
-- History of Changes:
--   OW57191 - LOB REPORTS TIMEOUT FROM REPORTING SYSTEM
--   OA09872 - Corrections to properly report on deleted objects.
--             Support includeDeleted option passed from report
--   OA11656 - When calling report_log_objects, the temp table #obj_temp now icluded a integer column
--             named "deleted'. This column will now contain a value of 1 if the resource is deleted.
--             The column value will be used to prefix resource names with a "*" for the report this
--             procedure supports. This was done to eliminate a Join to the link table which was returning
--             duplicated resources when a resource had both a deleted and undeleted link row.
--   PMR25706 - The mach and name column in #objtemp was increased in size from ObjName - NVARCHAR(54) to NVARCHAR(255)
--              This was required for deleted resources with names that were already at the max length of 54.
--              Attempting to pre-pend the '*' character onto deleted resource name would generate a "String
--              or bianry data would be truncated" error in the Reporting System WEB application.
--
CREATE PROCEDURE hist_LobEvents
  @id ObjID,
  @alertstates NVARCHAR(255),
  @priorities NVARCHAR(255),
  @begtime NVARCHAR(30),
  @endtime NVARCHAR(30),
  @eventtypes NVARCHAR(255),
  @orderopt NCHAR(1),
  @detail NVARCHAR(255),
  @IncludeChild TINYINT=1,
  @machineflag TINYINT=0,
  @nullstatepri NVARCHAR(50)='(Unknown)',
  @msgname NVARCHAR(255)='',
  @IncludeDeleted TINYINT=0,
  @extended_length_attr_filter NVARCHAR(255) -- DCR117
AS

SET NOCOUNT ON

--CREATE TABLE #objtemp (cid ClassID, id ObjID, mach ObjName NULL, name ObjName NULL, deleted INTEGER NULL)

CREATE TABLE #objtemp (cid ClassID, id ObjID, mach NVARCHAR(255) NULL, name NVARCHAR(255) NULL, deleted INTEGER NULL)

EXEC report_lob_objects @id, @IncludeChild, '', '#objtemp', @IncludeDeleted

EXEC report_addclassdata '#objtemp', 'name', 'name'

IF @machineflag=1
BEGIN

  EXEC report_addclassdata '#objtemp', 'mach', 'name', 'MACH'
  EXEC report_addclassdata '#objtemp', 'mach', 'name', 'UNIX'
  EXEC report_addclassdata '#objtemp', 'mach', 'name', 'NTSR'
  EXEC report_addclassdata '#objtemp', 'mach', 'name', 'HANT'
  EXEC report_addclassdata '#objtemp', 'mach', 'name', 'HAUX'
  
END

DECLARE @column_list NVARCHAR(300), @table_list NVARCHAR(255), @where_clause NVARCHAR(255)
DECLARE @orderby NVARCHAR(255), @where_clause2 NVARCHAR(255), @where_clause3 NVARCHAR(255)
-- dcr117 changed @whereClause from 255 to 510 in case the detail and extended attribute, each with 255 characters
DECLARE @where_clause4 NVARCHAR(600) 
DECLARE @orderbyfinal NVARCHAR(255)

SELECT @where_clause2='', @where_clause3='', @where_clause4=''

IF @machineflag=1
  --SELECT @column_list='machsys=isnull(o.mach,''(N/A)''),'
  SELECT @column_list='machsys=o.mach,'
ELSE
  SELECT @column_list=''

-- New Code 
IF @IncludeDeleted = 0
  SELECT @column_list = @column_list + 'objtype=o.cid, objname=o.name, obj_id = o.id, deleted=0, eventtype=e.cid,alertstate=isnull(ast._Desc,'''
ELSE
  SELECT @column_list = @column_list + 'objtype=o.cid, objname=o.name, obj_id = o.id, deleted=o.deleted, eventtype=e.cid,alertstate=isnull(ast._Desc,'''

SELECT @column_list = @column_list + @nullstatepri 
SELECT @column_list = @column_list + '''),priority=isnull(op._Priority,'''
--SELECT @column_list = @column_list + @nullstatepri + '''), ident=e.name,msgtext=e.detail,DATETIME=e.ctime'
SELECT @column_list = @column_list + @nullstatepri + '''), ident=isnull(e.name,''''),msgtext=isnull(e.detail,''''),DATETIME=e.ctime'

-- OA19932: since event table on primary db server does not have the extended_attr column (DCR 117), 
-- need to send in "" for it for customers running reporting system solely off primary db server
DECLARE @tableid INT
SELECT @tableid = id FROM sysobjects WHERE name = 'event'
IF EXISTS (SELECT * FROM syscolumns WHERE id = @tableid AND name = 'extended_attr')
  SELECT @column_list = @column_list + ',extended_attr=isnull(e.extended_attr,'''')' --DCR117
ELSE
  SELECT @column_list = @column_list + ',""'

--SELECT @table_list='#objtemp o,event e (index=event_PK),SixLevelPri_A op,AlertState_A ast,link c'
--SELECT @where_clause="o.cid=e.obj_cid AND o.id=e.obj_id AND ((o.cid=c.dst_cid AND o.id=c.dst_id) AND c.link_type = 'LOBL' ) and c.deleted = 0 AND op.id=e.priority AND ast.id=e.alertstate"

-- Code added by OW57191
SELECT @table_list='#objtemp o'
IF EXISTS (SELECT * FROM Object..sysobjects where name = 'event' and type = 'U')
   SELECT @table_list=@table_list+', event e (index=event_PK)'
ELSE
   BEGIN
     IF EXISTS (SELECT * FROM Object..sysobjects where name = 'event' and type = 'V')
        BEGIN
           DECLARE @vid INT
           DECLARE @text NVARCHAR(255)
           SELECT @vid=id FROM Object..sysobjects where name = 'event' and type = 'V'
           SELECT @text=text FROM Object..syscomments where id=@vid
           SELECT @vid=CHARINDEX('eventhist',@text)
           IF @vid > 0  -- BCP approach installed
              SELECT @table_list=@table_list+', History..eventhist e (index=event_PK)'
             ELSE       -- Linked server approach installed.
              SELECT @table_list=@table_list+', event e (index=event_PK)'
        END
     ELSE SELECT @table_list=@table_list+', event e (index=event_PK)'  -- event not 'U' or 'V' ... unknown type
   END

--SELECT @table_list=@table_list+', SixLevelPri_A op, AlertState_A ast, link c'
SELECT @table_list=@table_list+', SixLevelPri_A op, AlertState_A ast'

-- End Code added by OW57191
/* code replaced by OW57191
SELECT @table_list='#objtemp o,event e (index=event_PK),SixLevelPri_A op,AlertState_A ast'
*/

--SELECT @where_clause="o.cid=e.obj_cid AND o.id=e.obj_id and op.id=e.priority AND ast.id=e.alertstate AND o.cid=c.dst_cid AND o.id=c.dst_id"
SELECT @where_clause="o.cid=e.obj_cid AND o.id=e.obj_id and op.id=e.priority AND ast.id=e.alertstate"

IF @alertstates <> ''
BEGIN
  IF charindex(',',@alertstates) > 0
    SELECT @where_clause2=@where_clause2 + ' AND e.alertstate IN (' + @alertstates + ')'
  ELSE
    SELECT @where_clause2=@where_clause2 + ' AND e.alertstate=' + @alertstates
END
IF @priorities <> ''
BEGIN
  IF charindex(',',@priorities) > 0
    SELECT @where_clause2=@where_clause2 + ' AND e.priority IN (' + @priorities + ')'
  ELSE
    SELECT @where_clause2=@where_clause2 + ' AND e.priority=' + @priorities
END
IF @eventtypes<>''
BEGIN
  IF charindex(',',@eventtypes) > 0
    SELECT @where_clause2=@where_clause2 + ' AND e.cid IN (' + @eventtypes + ')'
  ELSE
    SELECT @where_clause2=@where_clause2 + ' AND e.cid=' + @eventtypes
END

-- filter on LOBL type to remove rows reported on multiple links
--SELECT @where_clause3=" AND e.ctime>=CONVERT(DATETIME,'" + @begtime + "',101) AND e.ctime<CONVERT(DATETIME,'" + @endtime + "',101)" + " and c.link_type = 'LOBL'"
SELECT @where_clause3=" AND e.ctime>=CONVERT(DATETIME,'" + @begtime + "',101) AND e.ctime<CONVERT(DATETIME,'" + @endtime + "',101)" 

IF @msgname <> ''
  --dcr117 the % signs are taken care of before the parameter is passed in.
  -- the search supports wildcard characters
  SELECT @where_clause3=@where_clause3 + ' AND e.name like N''%' + @msgname + '%'''
  -- SELECT @where_clause3=@where_clause3 + " AND e.name like N'" + @msgname + "'" -- APAR OA18672

IF @detail <> ''
  -- DCR117 the % signs are taken care of before the parameter is passed in.
  -- the search supports wild char character
  SELECT @where_clause4=' AND e.detail like N''%' + @detail + '%'''
  --SELECT @where_clause4 = " AND e.detail like '" + @detail + "' "  -- APAR OA18672

-- DCR117, the event search string should be converted to sql form before passing in
-- For example, if a user enters *tec event * into the search input in report,
-- the search string must be converted to %tec event% and pass this converted string into this procedure
--IF @extended_length_attr_filter <> ""
--   SELECT @where_clause4 = @where_clause4 + " AND e.extended_attr LIKE '" + @extended_length_attr_filter + "' "
IF @extended_length_attr_filter <> ''
   SELECT @where_clause4 = @where_clause4 + ' AND e.extended_attr LIKE N''%' + @extended_length_attr_filter + '%''' -- APAR OA18672

IF @orderopt = '1'
BEGIN
 SELECT @orderby='e.ctime'
  SELECT @orderbyfinal = "DATETIME"
END
ELSE
  BEGIN
    IF @machineflag=1
    BEGIN
      SELECT @orderby = 'o.mach,o.cid,o.name,e.ctime'
      SELECT @orderbyfinal = 'machsys,objtype,objname,DATETIME'
    END
    ELSE
    BEGIN
      SELECT @orderby = 'o.cid,o.name,e.ctime'
      SELECT @orderbyfinal = 'objtype,objname,DATETIME'
    END
  END

DECLARE @sql VARCHAR(2000)
DECLARE @sql2 VARCHAR(2000)
DECLARE @sql3 VARCHAR(2000)
DECLARE @sqlRemoveDups VARCHAR(2000), @sql4 VARCHAR(2000) -- DCR117
/*
IF @IncludeDeleted = 1
BEGIN
  SET @table_list = REPLACE(@table_list, '#objtemp', '#objtemp2')
  CREATE TABLE #objtemp2 (cid ClassID, id ObjID, mach ObjName NULL, name ObjName NULL, deleted TINYINT)

  INSERT #objtemp2 (cid, id, mach, name, deleted) 
  SELECT DISTINCT o.cid, o.id, o.mach, o.name, l.deleted FROM #objtemp AS o 
  JOIN link AS l ON l.dst_cid = o.cid AND l.dst_id = o.id AND l.link_type = 'LOBL'
END
*/

-- DCR117 create #final table with identity column, this is needed so that we can
-- remove duplicate rows later.
CREATE TABLE #final(
  row        INT IDENTITY(1,1),
  machsys    NVARCHAR(255) NULL,
  objtype    ClassID NULL, 
  objname    ObjName NULL, 
  obj_id     ObjID NULL, 
  deleted    INTEGER NULL,
  eventtype  ClassID NULL,
  alertstate NVARCHAR(16),
  priority   NVARCHAR(255) NULL,
  ident      ObjName NULL,
  msgtext    NVARCHAR(255) NULL,
  DATETIME   DATETIME NULL,
  extended_attr NTEXT NULL
)
IF @machineflag<>1
  ALTER TABLE #final DROP COLUMN machsys

--SET @sql = 'SELECT ' + @column_list + ' INTO #final FROM ' + @table_list + ' WHERE ' + @where_clause + @where_clause2 + @where_clause3 + @where_clause4 + ' ORDER BY ' + @orderby
SET @sql = 'INSERT INTO #final SELECT ' + @column_list + ' FROM ' + @table_list + ' WHERE ' + @where_clause + @where_clause2 + @where_clause3 + @where_clause4 + ' ORDER BY ' + @orderby

SET @sqlRemoveDups = 
'DELETE FROM #final
 WHERE EXISTS(
   SELECT row
     FROM #final fInner
     WHERE fInner.objtype = #final.objtype 
           AND fInner.objname = #final.objname 
           AND fInner.obj_id  = #final.obj_id 
           AND fInner.deleted  = #final.deleted
           AND fInner.eventtype  = #final.eventtype
           AND fInner.alertstate = #final.alertstate
           AND fInner.priority   = #final.priority
           AND fInner.ident      = #final.ident
           AND fInner.msgtext    = #final.msgtext
           AND fInner.DATETIME   = #final.DATETIME
           AND fInner.row < #final.row'

IF @machineflag<>1
  SET @sqlRemoveDups = @sqlRemoveDups + ')'
ELSE
  SET @sqlRemoveDups = @sqlRemoveDups + ' AND fInner.machsys = #final.machsys)'

IF @machineflag=1
BEGIN

SET @sql2 = CHAR(13) + 
"IF EXISTS(SELECT * FROM #final WHERE machsys IS NULL)
BEGIN

  DECLARE @position INT
  DECLARE @mach NVARCHAR(255)
  DECLARE @idpath NVARCHAR(1375)
  DECLARE @resourcepath NVARCHAR(1375)
  DECLARE @obj_cid ClassID
  DECLARE @obj_id ObjID

  CREATE table #elements ( position int, element NVARCHAR(400))

  DECLARE final_cursor CURSOR FOR SELECT objtype, obj_id FROM #final WHERE machsys IS NULL
  OPEN final_cursor
  FETCH NEXT FROM final_cursor INTO @obj_cid, @obj_id
  WHILE @@FETCH_STATUS = 0
  BEGIN

    SELECT @idpath = idpath,@resourcepath = resourcepath FROM ObjPathCache WHERE cid = @obj_cid and id = @obj_id
    EXEC asisp_splitString @idpath,'/'

    SELECT @position = position FROM #elements WHERE element like 'MACH:%'

    DELETE #elements

    EXEC asisp_splitString @resourcepath,'/'

    SELECT @mach = element FROM #elements WHERE position = @position
    UPDATE #final SET machsys = @mach WHERE objtype = @obj_cid AND obj_id = @obj_id

    UPDATE #final SET machsys = '(N/A)' WHERE machsys IS NULL     -- defect 117022    

    FETCH NEXT FROM final_cursor INTO @obj_cid, @obj_id

  END
  CLOSE final_cursor
  DEALLOCATE final_cursor

END
"
  -- mark deleted objects with an *
  -- DCR117 after updating #final with objname='*', need to remove duplicate rows manually before
  -- selecting data instead of using SELECT DISTINCT.  This way we can include the extended_attribute into the select statement
  --SET @sql3 = CHAR(13) + "UPDATE #final SET objname = '*' + objname WHERE deleted = 1 SELECT DISTINCT machsys, objtype, objname, deleted, eventtype, alertstate, priority, ident, msgtext, DATETIME FROM #final ORDER BY " + @orderbyfinal
  SET @sql3 = CHAR(13) + "UPDATE #final SET objname = '*' + objname WHERE deleted = 1 " 
  EXEC(@sql + @sql2 + @sql3)
  --PRINT 'SQL with machsys := ' + @sql + @sql2 + @sql3

  -- remove duplicate rows from #final
  EXEC(@sqlRemoveDups)

  -- select data to return
  SET @sql4 = "SELECT machsys, objtype, objname, deleted, eventtype, alertstate, priority, ident, msgtext, DATETIME, extended_attr FROM #final ORDER BY " + @orderbyfinal
  EXEC(@sql4)
 
END
ELSE
BEGIN
  --PRINT 'SQL without machsys := ' + @sql + @sql3
  -- mark deleted objects with an *
  -- DCR117 - same as with machine sql above
  --SET @sql3 = CHAR(13) + "UPDATE #final SET objname = '*' + objname WHERE deleted = 1 SELECT DISTINCT objtype, objname, deleted, eventtype, alertstate, priority, ident, msgtext, DATETIME FROM #final ORDER BY " + @orderbyfinal
  SET @sql3 = CHAR(13) + "UPDATE #final SET objname = '*' + objname WHERE deleted = 1"
  EXEC(@sql + @sql3)

  -- remove duplicate rows from #final
  EXEC(@sqlRemoveDups)

  -- select data for return
  SET @sql4 = "SELECT objtype, objname, deleted, eventtype, alertstate, priority, ident, msgtext, DATETIME, extended_attr FROM #final ORDER BY " + @orderbyfinal
  EXEC(@sql4)

END

SET NOCOUNT OFF
GO