-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
--  Copyright IBM Corp. 2007 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive/db/sql/tracks/oa19918.sql, tbsm_base, tbsm_base31e
-- SID: 1.2
-- Date Modified: 07/02/28 19:52:01
--
-- For APAR OA19918.
-- Script to clear out obsolete StagedEEVT records, older than 1 day.
--
SET NOCOUNT ON

USE Object
GO

DECLARE @dt datetime
DECLARE @rc int
DECLARE @rowcount int
DECLARE @error int
DECLARE @msg nvarchar(100)

SELECT @rowcount = 1

SELECT @dt = dateadd ( dd, -1, getdate())

SELECT @rc = COUNT(*) FROM StagedEEVT WHERE ctime < @dt AND timestamp NOT IN 
             ( SELECT timestamp from StagedEXCP with (NOLOCK)
               UNION
               SELECT timestamp FROM StagedMESG WITH (NOLOCK)
               UNION
               SELECT timestamp FROM UnhandledStagedEXCP WITH (NOLOCK)
               UNION
               SELECT timestamp FROM UnhandledStagedMESG WITH (NOLOCK)
               UNION
               SELECT timestamp FROM StagedEXCPInProgress WITH (NOLOCK)
               UNION
               SELECT timestamp FROM StagedEEVTInProgress WITH (NOLOCK)
             )

IF ( @rc = 0 )
  RAISERROR('There are no obsolete records from the StagedEEVT table to delete.',3,1) WITH NOWAIT

WHILE ( @rc > 0 AND @rowcount > 0 )
BEGIN
  SET ROWCOUNT 1000
  DELETE FROM StagedEEVT WHERE ctime < @dt AND timestamp NOT IN
  ( SELECT timestamp FROM StagedEXCP WITH (NOLOCK)
    UNION
    SELECT timestamp FROM StagedMESG WITH (NOLOCK)
    UNION
    SELECT timestamp FROM UnhandledStagedEXCP WITH (NOLOCK)
    UNION
    SELECT timestamp FROM UnhandledStagedMESG WITH (NOLOCK)
    UNION
    SELECT timestamp FROM StagedEXCPInProgress WITH (NOLOCK)
    UNION
    SELECT timestamp FROM StagedEEVTInProgress WITH (NOLOCK)
)
  SELECT @error = @@error, @rowcount = @@rowcount

  IF @@error <> 0
  BEGIN
    RAISERROR('ERROR: An error occured clearing the StagedEEVT table.',3,1) WITH NOWAIT
    SET @rc=0
  END
  ELSE
  BEGIN
    SELECT @msg = 'Deleted ' + convert(nvarchar(10),@rowcount) + ' from StagedEEVT, ' + convert(nvarchar(10),@rc-@rowcount) + ' rows to go.'
    RAISERROR('%s',0,1,@msg) WITH NOWAIT
    SELECT @rc = @rc - @rowcount
  END
END

SET ROWCOUNT 0

GO

